/****************************************************************************
*
* File:             Cme.c
* Author:           Nikolaos Radouniklis
* Date:             07/02/96
* Last changed:     07/09/96
* Description:      Session Manager manages the entire CME session
*                   Provides interfaces to: 
*                   1) the User Interface 
*                   2) the QoS Mapper & Controler
*					3) the Service Manager
*					4) the Connection Manager
*
* Comment:			It contains a customized version of the 
*					Tcl_AppInit procedure with new Tcl/Tk commands:
*					-
****************************************************************************/


/* system includes */

#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <msql.h>
#include <tcl.h>
#include <tk.h>

/* my includes */
#include "MS_Types.h"
#include "Constants.h"
#include "NewTclCommands.h"
#include "ConnectionManager.h"
#include "Database.h"
#include "QoSMapper.h"
#include "QoSController.h"


/* prototypes */


/*
 * The following variable is a special hack that is needed in order for
 * Sun shared libraries to be used for Tcl.
 */
extern int matherr();
int *tclDummyMathPtr = (int *) matherr;





/* global variables */
int sock;               /* socket descriptor for CME database access 	*/
Tcl_Interp *G_interp;   /* global tcl interpreter for user interface	*/
Tcl_Interp *interp;     /* global Tcl interpreter for ms 				*/
InvitationLetterStruct InvitationLetter; /* global InvitationLetter 	*/
int G_Sock;		/* Socket descriptor for CM */






/****************************************************************************
*
* Procedure:		main 
* Param:
* Result:			(-1) OK, 0 Error 
* Date:             07/02/96
* Last changed:     07/02/96
* Description:      
* 
* Comment:       
* 
****************************************************************************/
int main (int argc, char *argv[])
{	



	/* What does Tk_Main ?												*/
	/* 1) Create Tcl interpreter										*/
	/* 2) Parse some window related command arguments 					*/
	/* 3) Create main window for the application (Tk_CreateMainWindow)	*/
	/* 4) Call Tcl_AppInit provided by this application					*/
	/* 5) Read a script													*/
	/* 6) Enter an event loop to process events and interactive comm. 	*/

	Tk_Main (argc, argv, Tcl_AppInit);


    return (-1);
}






/****************************************************************************
*
* Procedure:        Tcl_AppInit 
* Param: 
* Result:           Returns a standard Tcl completion code and leaves an error
*					message in interp->result if an error occurs
* Date:             07/02/96
* Last changed:     07/02/96
* Description:      This procedure performs application-specific initialization
*					Most applications, especially those that incorporate additional
*					packages, will have their own version of this procedure.
*
* Comment:       	Create Commands:
*					- SMStartMS (Session Manager Start Media Service)
*
****************************************************************************/


int Tcl_AppInit(interp)
    Tcl_Interp *interp;		/* Interpreter for application. */
{


    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    if (Tk_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }

	/* set global variable G_interp */
	G_interp = interp;

    /* Call Tcl_CreateCommand for application-specific commands, if			*/
    /* they weren't already created by the init procedures called above.	*/
	RegisterNewTclCommands (interp);



  	/* Specify a user-specific startup file to invoke if the application		*/
    /* is run interactively.  Typically the startup file is "~/.apprc"			*/
    /* where "app" is the name of the application.  If this line is deleted		*/
    /* then no user-specific startup file will be run under any conditions.		*/
	/*
    tcl_RcFileName = 
	*/

	Tcl_EvalFile ( interp, "./UI.tcl");
	Tcl_Eval (interp, "InitCM");
    /* Initialize server listening for invitation calls                 */
    WaitForInvitation ( atoi (DEF_PORT));

	/* invoke Tk_MainLoop again */
	Tk_MainLoop ();

	/* Return from Tcl_AppInit */
	return TCL_OK;

}





