/****************************************************************************
*
* File:             ConnectionManager.h
* Author:           Nikolaos Radouniklis
* Date:             07/10/96
* Last changed:     07/10/96
* Description:      Connection Manager for the entire CME session
*                   Provides functionality for following tasks:
*                   1) Invitation
*                   2) DisConnection
* Comment:
*
****************************************************************************/

#ifndef _connectionmanager_h
#define _connectionmanager_h

#include "MS_Types.h"
#include <netinet/in.h>
#include <tcl.h>
#include <tk.h>



/****************************************************************************
*
* Procedure:        ChechkParticipant
* Param:            [in]  ParticipantHost: (e.g icsib8.icsi.berkeley.edu)
*                   [in]  Port: Portnumber where participant is listening to
*                   [in]  InvitationLetter: (structure with invitation info)
* Result:           CONNECT_OK, CONNECT_FAIL
* Date:             07/09/96
* Last changed:     07/09/96
* Description:      Check if Participant server process is running
*
****************************************************************************/
int CheckParticipant ( char* ParticipantHost, int Port );




/****************************************************************************
*
* Procedure:        InviteParticipant
* Param:            [in]  ParticipantHost: (e.g icsib8.icsi.berkeley.edu)
*                   [in]  Port: Portnumber where participant is listening to
*                   [in]  InvitationLetter: (structure with invitation info)
* Result:           (-1) OK; 0 ERROR
* Date:             07/09/96
* Last changed:     07/09/96
* Description:
*
****************************************************************************/
int InviteParticipant ( char* ParticipantHost, int Port, InvitationLetterStruct *InvitationLetter );



/****************************************************************************
*
* Procedure:        SendInvitationLetter
* Param:            [in]  sockd: (Socket descriptor to participant)
*                   [in]  InvitationLetter: (structure with invitation info)
* Result:
* Date:             07/09/96
* Last changed:     07/09/96
* Description:      Send the invitation to socket descriptor
*                   which connects the session initiator to the participant
*
****************************************************************************/
int SendInvitationLetter ( int sockd, InvitationLetterStruct *InvitationLetter );




/****************************************************************************
*
* Procedure:        ReadInvitationLetter
* Param:            [in]  sockd: (Socket descriptor to participant)
*                   [in]  InvitationLetter: (structure with invitation info)
* Result:           (0) EOF is reached; (-1) Data read
* Date:             07/09/96
* Last changed:     07/09/96
* Description:      Receive the invitation from the session initiator
*
****************************************************************************/
int ReadInvitationLetter ( int sockd, InvitationLetterStruct *InvitationLetter );




/****************************************************************************
*
* Procedure:        WaitForInvitation
* Param:            [in]  port: Portnumber to listen
*
* Result:           (-1) OK, 0 ERROR
* Date:             07/09/96
* Last changed:     07/09/96
* Description:      Wait for the InvitationLetter
*                   Set the global Socket Descriptor G_Sock
*                   before invoking the File Event Handler
*
****************************************************************************/
int WaitForInvitation (int port);




/****************************************************************************
*
* Procedure:        ProcessInvitation
* Param:            [in] clientdata
*                   [in] mask
* Result:
* Date:             07/10/96
* Last changed:     07/10/96
* Description:      Read the incoming invitation request
*                   !Global variable G_Sock
*
****************************************************************************/
void ProcessInvitation (ClientData clientdata, int mask);






#endif
