/****************************************************************************
*
* File:             Database.h
* Author:           Nikolaos Radouniklis
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      database API functions for CME-DB
*
*
****************************************************************************/


#ifndef _database_h
#define _database_h


/****************************************************************************
*
* Function:         DBConnect
* Param:            [in] DBName
* Returns:          [res] sock (Socket descriptor)
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      Connects to the CME database
*
****************************************************************************/

int DBConnect (char *DBName);


/*****************************************************************************
*
* Function:         DBRelease
* Param:            [in] sock (Socket connection of database)
* Returns:          -1 (OK); 0 (Error)
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      Connects to the CME database
* 
****************************************************************************/
 

int DBRelease (int sock);



/****************************************************************************
*
* Procedure:        error_handler
* Date:             05/30/96
* Last changed:     06/02/96
* Description:      Handle error in intialisation
* Parameters:
*
****************************************************************************/
 
int error_handler ();




/****************************************************************************
*
* Function:         DBClearTable
* Param:            [in] sock (Socket connection of database)
*                   [in] table (Name of the table to be cleared)
* Returns:          -1 (OK); 0 (Error)
* Date:             06/04/96
* Last changed:     06/04/96
* Description:      Connects to the CME database
*
****************************************************************************/

int DBClearTable (int sock, char *Table);



/****************************************************************************
*
* Function:         DBAverage
* Param:            [in] sock (Socket connection to database)
*                   [in] table (Name of the table)
*                   [in] attr  (Name of the attribute)
*                   [in] samples (no of samples to be considered)
*                   [in] Condition (additional condition for SQL-Query) or NULL
*                   [out] Avg (average value of attr over samples)
* Returns:          (-1) OK; (0) Error;
* Date:             06/07/96
* Last changed:     06/26/96
* Description:      Computes the average value of the attribute <attr>
*                   in the table <table>
*
****************************************************************************/

int DBAverage (int sock, char *Table, char *Attr, int Samples, char* Condition, float *Avg);



/****************************************************************************
*
* Function:         DBGetAttr
* Param:            [in] sock (Socket connection to database)
*                   [in] table (Name of the table)
*                   [in] attr  (Name of the attribute)
*                   [in] Condition (additional condition for SQL-Query) or NULL
*                   [out] Value (string value)
* Returns:          (-1) OK; (0) Error;
* Date:             06/26/96
* Last changed:     06/26/96
* Description:      retrieve the attribute value <Value> of <Attr>
*                  
*
****************************************************************************/
 
int DBGetAttr (int sock, char *Table, char *Attr, char *Condition, char *Value);





/****************************************************************************
*
* Function:         DBGetLastRow
* Param:            [in] sock (Socket connection to database)
*                   [in] Table (Name of the table)
*                   [in] attr  (Name of the attribute)
* Returns:          Last (Counter Value of last inserted row)
* Date:             06/06/96
* Last changed:     06/06/96
* Description:      (--> Returns)
*   
****************************************************************************/
 
int DBGetLastRow (int sock, char *Table, char *Attr);



/****************************************************************************
*
* Function:         DBFieldIsType
* Param:            [in] sock (Socket connection to database)
*                   [in] Table (Name of the table)
*                   [in] attr  (Name of the attribute)
* Returns:          INT_TYPE, CHAR_TYPE, REAL_TYPE (type of attribute)
* Date:             06/07/96
* Last changed:     06/07/96
* Description:      Retrieves the attribute type of attr in Table
*
****************************************************************************/
 
int DBFieldIsType (int sock, char *Table, char *Attr);



/****************************************************************************
*
* Procedure:        drop_tables (option -d)
* Date:             05/29/96
* Last changed:     07/18/96
* Description:      drop the CME database tables
* Parameters:
*
****************************************************************************/
 
int drop_tables();



/****************************************************************************
*
* Procedure:        build_tables
* Date:             05/29/96
* Last changed:     07/18/96
* Description:      build the CME database tables
* Parameters:
*
****************************************************************************/
 
int build_tables ();


/****************************************************************************
*
* Procedure:        DBGetName
* Date:             07/18/96
* Last changed:     07/18/96
* Description:      concatenates CME_Prefix && Host --> DBName
* Parameters:       --
* 
* Returns:          DBName   
*
****************************************************************************/
 
char *DBGetName ();

/****************************************************************************
*
* Function:         DBCreate
* Param:
* Returns:          (-1) OK; (0) Error
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      Creates a CME database
*
****************************************************************************/
 
int DBCreate ();


/****************************************************************************
*
* Function:         DBDestroy
* Param:
* Returns:          (-1) OK; (0) Error
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      Destroys a CME database
*
****************************************************************************/
 
int DBDestroy ();

/**************************************************************************** 
*
* Function:         DBAvg
* Param:            [in] sock (Socket connection to database)
*                   [in] table (Name of the table)
*                   [in] attr  (Name of the attribute)
*                   [in] Condition (additional condition for SQL-Query) or NULL
*                   [out] Avg (average value of attr )
* Returns:          (-1) OK; (0) Error;
* Date:             07/22/96
* Last changed:     07/22/96
* Description:      Computes the average value of the attribute <attr>
*                   in the table <table>
*
****************************************************************************/

int DBAvg (int sock, char *Table, char *Attr, char* Condition, float *Avg);



/**************************************************************************** 
*
* Function:         DBRowsExist
* Param:            [in] sock (Socket connection to database)
*                   [in] SQLstatement
* Returns:          NumRows ( number of rows for SQL statement)
* Date:             07/25/96
* Last changed:     07/25/96
* Description:      Checks if rows exist for a given SQLstatement
*                   
*
****************************************************************************/
 
int DBRowsExist (int sock, char *SQLstatement );


#endif
