/****************************************************************************
*
* File:             NewTclCommands.c
* Author:           Nikolaos Radouniklis
* Date:             07/11/96
* Last changed:     07/18/96
* Description:      Creates New Tcl Commands for the Tcl UI
*                   Provides commands for:
*                   1) the Session Management
*					2) the Connection Management
*
* Comment:			
*					
****************************************************************************/


/* system includes */

#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <signal.h>
#include <msql.h>
#include <tcl.h>
#include <tk.h>

/* my includes */
#include "MS_Types.h"
#include "Constants.h"
#include "ConnectionManager.h"
#include "ServiceManager.h"
#include "QoSMapper.h"
#include "Database.h"
#include "inet.h"
#include "QoSController.h"
#include "util.h"


/* global structure InvitationLetter */
extern InvitationLetterStruct InvitationLetter;
extern int sock; /* Socket descriptor for database */
extern int G_Sock;




/* prototypes */
int RegisterNewTclCommands (Tcl_Interp *interp);
int Tclgethostname ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int Tclgetusername ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int Tclgetdefault ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int Tclfillinvitation ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int Tclgetinvletter ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int Tclstartms ( ClientData clientData, Tcl_Interp *interp, int argc, char *argv[] );
int Tclstopms ( ClientData clientData, Tcl_Interp *interp, int argc, char *argv[] );
int Tclcalcqual ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int Tclcmestartup ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int Tclcmecleanup ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int Tclwritewish ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int TclbuildDBsockets ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int TcldestroyDBsockets ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int Tclqoscontrol ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int Tclgetqual ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );
int Tclinitwish ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] );





/****************************************************************************
*
* Procedure:        RegisterNewTclCommands
* Param:
* Result:           (-1) OK, 0 Error
* 
* Date:             07/11/96
* Last changed:     07/11/96
* Description:      Register the new Tcl Commands
*
****************************************************************************/

int RegisterNewTclCommands ( Tcl_Interp *interp )
{
	
	Tcl_CreateCommand (interp, "gethostname", Tclgethostname, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "getusername", Tclgetusername, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "getdefault", Tclgetdefault, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "fillinvitation", Tclfillinvitation, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "getinvletter", Tclgetinvletter, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "startms", Tclstartms, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "stopms", Tclstopms, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "calcqual", Tclcalcqual, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "cmestartup", Tclcmestartup, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "cmecleanup", Tclcmecleanup, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "writewish", Tclwritewish, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "buildDBsockets", TclbuildDBsockets, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "destroyDBsockets", TcldestroyDBsockets, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "qoscontrol", Tclqoscontrol, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "getqual", Tclgetqual, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand (interp, "initwish", Tclinitwish, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	return (-1);
}



/****************************************************************************
*
* Procedure:        Tclgethostname
* Param:			
* Result:           TCL_OK
* interp->result:	hostname
* Date:             07/11/96
* Last changed:     07/11/96
* Description:      Retrieve the hostname by calling the corr. C-procedure
*
****************************************************************************/

int Tclgethostname ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{
	char Host[MAX_CHARBUFFER];
	
	gethostname (Host, MAX_CHARBUFFER -1);
	strcpy ((*interp).result, Host);
	
	return TCL_OK;
}
	

/****************************************************************************
*
* Procedure:        Tclgetusername
* Param:            
* Result:           TCL_OK
* interp->result:   username
* Date:             07/11/96
* Last changed:     07/11/96
* Description:      Retrieve the username of the active process
*
****************************************************************************/
 
int Tclgetusername ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{
    char User[MAX_CHARBUFFER]; 
    
    cuserid (User);
    strcpy ((*interp).result, User);
     
    return TCL_OK; 
}



/****************************************************************************
*
* Procedure:        Tclgetdefault
* Param:			[in] attr: (Attributname) "vicport vatport wbport address port"
*					
* Result:           TCL_OK, TCL_ERROR
* interp->result:   value
* Date:             07/11/96
* Last changed:     07/11/96
* Description:      Retrieve the default value of atrribute <attr>
*
****************************************************************************/

int Tclgetdefault ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{

	if (argc != 2) 
	{
		strcpy ((*interp).result, "wrong # args" );
		return TCL_ERROR; 
	}
	
	if  (strcmp ( argv[1], "vicport" ) == 0)
		strcpy ((*interp).result, DEF_VIC_PORT );
	else if (strcmp ( argv[1], "vatport" ) == 0) 
		strcpy ((*interp).result, DEF_VAT_PORT );
	else if (strcmp ( argv[1], "wbport" ) == 0)
		strcpy ((*interp).result, DEF_WB_PORT );
	else if (strcmp ( argv[1], "address" ) == 0)
		strcpy ((*interp).result, DEF_MULTICAST_ADDRESS );
	else if (strcmp ( argv[1], "port" ) == 0)
		strcpy ((*interp).result, DEF_PORT );

	return TCL_OK;


}

/****************************************************************************
*
* Procedure:        Tclfillinvitation
* Param:			[in] address (Multicast address)
*					[in] vicport
*					[in] vatport
*					[in] wbport
*					[in] ttl
*					[in] plist (Participantlist)
*					[in] invitation port
*					[in] vicqual
*					[in] vatqual
*					[in] wbqual
*
* Result:           TCL_OK
* interp->result:   FailedAddresses 
* Date:             07/12/96
* Last changed:     07/22/96
* Description:      1) Creates invitation structure
*					2) Checks if participants are available
*
* Comment:			Uses global structure InvitationLetter
*
****************************************************************************/

int Tclfillinvitation ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{

	char tempbuf [MAX_PARTICIPANT_LIST]; /* holds max. participant length */
	char *token;
	int retval; 
	int i;
	char PAddress[MAX_PARTICIPANT_LENGTH]; 	/* e.g. pop.icsi 			*/
	char PAddress2[MAX_PARTICIPANT_LENGTH];	/* e.g. 128.32.123.12		*/ 
	char PName[MAX_PARTICIPANT_LENGTH];		/* e.g. nikolaus			*/ 
	char FailedAddresses [MAX_PARTICIPANT_LIST]; /* list of failed addresses */


	/* Copy commandline arguments into InvitationLetter */
	/* 1. Multicast address */
	strcpy (InvitationLetter.Address, argv[1]);
	/* 2. vic port */
	if ( strcmp (argv[2], "") != 0)
		InvitationLetter.VicPort = atoi (argv[2]);
	else
		InvitationLetter.VicPort = 0;
	/* 3. vat port */
	if (strcmp (argv[3], "") != 0)
		InvitationLetter.VatPort = atoi (argv[3]);
	else
		InvitationLetter.VatPort = 0;
	/* 4. wb port */
	if (strcmp (argv[4], "") != 0)
		InvitationLetter.WbPort = atoi (argv[4]);
	else
		InvitationLetter.WbPort = 0;
	/* 5. ttl (Time to Live) */
	InvitationLetter.TTL = atoi (argv[5]);	
	/* 6. Participant List */
	strcpy ( tempbuf, argv[6] );
	i=0;
	token = strtok ( tempbuf, " " );
    while (token !=NULL)
    {
        strcpy ( InvitationLetter.Participant[i], token );
        token = strtok (NULL, " ");
        i+=1;
    }
	strcpy ( InvitationLetter.Participant[i], "" );

	/* 6b. Participant2 List (e.g. nikolaus@128.32.32.127)  */
	i =0;
	while ( strcmp (InvitationLetter.Participant[i], "") != 0 )
    {
        /* Participant format is: <name>@<address>  */
        /* Extract <name> from Participant  	    */
        strcpy (tempbuf, InvitationLetter.Participant[i]);
        token = strtok ( tempbuf, "@" );
        strcpy (PName, token);
        token = strtok ( NULL, "@" );
        strcpy (PAddress, token);
		
		strcpy (PAddress2, InetNtoa(LookupHostAddr(PAddress)) );
		sprintf (InvitationLetter.Participant2[i], "%s@%s", PName, PAddress2 );

        i+=1;                                                                     
    }
	strcpy ( InvitationLetter.Participant2[i], "" );
	
	/* 7. Invitation Port */
	InvitationLetter.Port = atoi (argv[7]);


	/* 8. 9. 10.  viqual vatqual wbqual */
	for (i=0; i<MAX_PARTICIPANTS; i++)
	{
		InvitationLetter.VicQoS[i] = atoi (argv[8]);
		InvitationLetter.VatQoS[i] = atoi (argv[9]);
		InvitationLetter.WbQoS[i] = atoi (argv[10]);
	}



	/* Check if participants are available */
	i = 1;
	strcpy (FailedAddresses, "");
	while ( strcmp (InvitationLetter.Participant[i], "") != 0 )
	{
		/* Participant format is: <name>@<address> */
		/* Extract <address> from Participant	   */
		strcpy (tempbuf, InvitationLetter.Participant[i]);
		token = strtok ( tempbuf, "@" );
		token = strtok ( NULL, "@");
		strcpy (PAddress, token);

		retval = CheckParticipant ( PAddress, InvitationLetter.Port );
		/* generate the FailedAddresses list */
		if (retval == CONNECT_FAIL)
		{
			strcat ( FailedAddresses, InvitationLetter.Participant[i]);
			strcat ( FailedAddresses, "\n");
		} 

		i+=1;
	}

	/* return if FailedAddresses list is not empty */
 	if ( strcmp (FailedAddresses, "") != 0 )
	{
		strcpy ((*interp).result, FailedAddresses );
		return (TCL_OK);
	}


	/* proceed with invitation ... after having checked all participants*/
	i = 1;
	while ( strcmp (InvitationLetter.Participant[i], "") != 0 )
    {
        /* Participant format is: <name>@<address> */
        /* Extract <address> from Participant      */
        strcpy (tempbuf, InvitationLetter.Participant[i]);
        token = strtok ( tempbuf, "@" );
        token = strtok ( NULL, "@");
        strcpy (PAddress, token);
 
        InviteParticipant ( PAddress, InvitationLetter.Port, &InvitationLetter );
        
        i+=1;
    }


	strcpy ((*interp).result, "");
	return TCL_OK;
}




/****************************************************************************
*
* Procedure:        Tclgetinvletter
* Param:			[in] Attribute ( e.g. plist, address, vicport, vatport, wbport, ttl, port )
* 
* Result:           TCL_OK
* interp->result:   attribute value
* Date:             07/15/96
* Last changed:     07/15/96
* Description:      Retrieve the attributr value of the invitation letter structure
*
* Comment:			InvitationLetter = global structure!
****************************************************************************/

int Tclgetinvletter ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{

	int i = 0;
    char PList[MAX_PARTICIPANT_LIST];
    char tempbuf[MAX_CHARBUFFER];
    if (argc != 2)
    {
        strcpy ((*interp).result, "wrong # args" );
        return TCL_ERROR;
    }

	/* retrieve now the corresponding attribute */
	if ( strcmp ( argv[1], "plist" ) == 0)
	{
		PList[0] = '\0';
		while ( strcmp ( InvitationLetter.Participant[i], "" ) != 0 )
		{
			strcat (PList, InvitationLetter.Participant[i]);
			strcat (PList, " ");
			i+=1;
		}
		strcpy ((*interp).result, PList );

	}
	else if ( strcmp ( argv[1], "address" ) == 0)
		strcpy ((*interp).result, InvitationLetter.Address );
	else if ( strcmp ( argv[1], "vicport" ) == 0)
	{
		sprintf ( tempbuf, "%d", InvitationLetter.VicPort);
		strcpy ((*interp).result, tempbuf );
	}
	else if ( strcmp ( argv[1], "vatport" ) == 0)
	{
    	sprintf ( tempbuf, "%d", InvitationLetter.VatPort); 
        strcpy ((*interp).result, tempbuf );
	}
	else if ( strcmp ( argv[1], "wbport" ) == 0)
	{
    	sprintf ( tempbuf, "%d", InvitationLetter.WbPort); 
        strcpy ((*interp).result, tempbuf );
	}
	else if ( strcmp ( argv[1], "ttl" ) == 0)
	{
    	sprintf ( tempbuf, "%d", InvitationLetter.TTL); 
        strcpy ((*interp).result, tempbuf );
	}
	else if ( strcmp ( argv[1], "port" ) == 0)
	{
    	sprintf ( tempbuf, "%d", InvitationLetter.Port); 
        strcpy ((*interp).result, tempbuf );
	}



    return TCL_OK;
}





/****************************************************************************
*
* Procedure:        Tclstartms
* Param:            argv[1]: == MSName (vic, vat, wb )
* 
* Result:           TCL_OK
* interp->result:   
* Date:             07/16/96
* Last changed:     07/16/96
*					
* Description:      Start the in InvitationLetter specified media services
*
* Comment:          InvitationLetter = global structure!
*					sock: global socket descriptor for CME database
****************************************************************************/

int Tclstartms ( ClientData clientData, Tcl_Interp *interp, int argc, char *argv[] )
{

	 
    char *vicArgList[MS_MAX_ARGS];
    char *vatArgList[MS_MAX_ARGS];
	char *wbArgList [MS_MAX_ARGS];
	char tempbuf[MAX_CHARBUFFER];
	int i;

	
	/* Start Media Services if selected (Read for this InvitationLetter) */
	if ( (strcmp(argv[1], VIC) == 0) && ( InvitationLetter.VicPort != 0 ) )
	{

		/* Allocate first place for command arguments 		*/
		for ( i=0; i<MS_MAX_ARGS; i++ )
			vicArgList[i] = (char *) malloc ( MAX_CHARBUFFER );
		/* Generate the appropriate Command line arguments 	*/
		strcpy ( vicArgList[0], VIC_EXEC );
		sprintf (tempbuf, "-t %d", InvitationLetter.TTL);
		strcpy ( vicArgList[1], tempbuf );
		sprintf (tempbuf, "%s/%d", InvitationLetter.Address, InvitationLetter.VicPort );
		strcpy ( vicArgList[2], tempbuf );
		vicArgList[3] = NULL;
 
		/* Start now the VIC MS */ 
    	StartMS ( VIC , VIC_COMMAND, vicArgList);
	}


    if ( (strcmp(argv[1], VAT) == 0) && ( InvitationLetter.VatPort != 0 ) )
    {

		/* Allocate first place for command arguments 		*/
		for ( i=0; i<MS_MAX_ARGS; i++ )
			vatArgList[i] = (char *) malloc ( MAX_CHARBUFFER );
		/* Generate the appropriate Command line arguments */ 
        strcpy ( vatArgList[0], VAT_EXEC );   
		strcpy ( vatArgList[1], "-r" );
        sprintf (tempbuf, "-t %d", InvitationLetter.TTL); 
        strcpy ( vatArgList[2], tempbuf );  
        sprintf (tempbuf, "%s/%d", InvitationLetter.Address, InvitationLetter.VatPort ); 
        strcpy ( vatArgList[3], tempbuf ); 
        vatArgList[4] = NULL; 

		/* Invoke now the VAT MS */
    	StartMS ( VAT , VAT_COMMAND, vatArgList);
	}

	if ( (strcmp(argv[1], WB) == 0) && ( InvitationLetter.WbPort != 0 ) )
    {
		/* Allocate first place for command arguments       */
        for ( i=0; i<MS_MAX_ARGS; i++ )
            wbArgList[i] = (char *) malloc ( MAX_CHARBUFFER );
		/* Generate the appropriate Command line arguments */
        strcpy ( wbArgList[0], WB_EXEC );
		sprintf( wbArgList[1], "-t" );
		sprintf( wbArgList[2], "%d", InvitationLetter.TTL );
        sprintf (wbArgList[3], "%s/%d", InvitationLetter.Address, InvitationLetter.WbPort );
        wbArgList[4] = NULL;

		/* Start now the WB MS */
 	  	StartMS ( WB , WB_COMMAND, wbArgList);
    } 



    return TCL_OK;
}



/****************************************************************************
*
* Procedure:        Tclstopms
* Param:			argv[1] == MSName
*
* Result:           TCL_OK
* interp->result:
* Date:             07/16/96
* Last changed:     07/16/96
* Description:      Stop the media services (... and the corresponding monitors)
*
* Comment:          InvitationLetter = global structure!
*                   sock: global socket descriptor for CME database
****************************************************************************/

int Tclstopms ( ClientData clientData, Tcl_Interp *interp, int argc, char *argv[] )
{

    /* Stop the Media Services which have been started */
	if ( (strcmp(argv[1], VIC) == 0) && ( InvitationLetter.VicPort != 0 ) )
    	StopMS (VIC);
	if ( (strcmp(argv[1], VAT) == 0) && ( InvitationLetter.VatPort != 0 ) )
    	StopMS (VAT);
	if ( (strcmp(argv[1], WB) == 0) && ( InvitationLetter.WbPort != 0 ) )
		StopMS (WB);

                  
    return TCL_OK;
}



/****************************************************************************
*
* Procedure:        Tclcalcqual
* Param:            argv[1]:MSName  (Name of MS)
*					argv[2]:ParticipantIndex
*					argv[3]:Samples (Number of samples to be considered)
*					
* Result:           TCL_OK
* (interp->result)  UserQuality 
*
* Date:             07/16/96
* Last changed:     07/18/96
* Description:      Calculates the user quality of the ms application
*					Retrieves Application Parameters and  maps 
*
*
****************************************************************************/
 
int Tclcalcqual ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{
 
    static char MSName[MAX_CHARBUFFER];
	static int ParticipantIndex;
	static char Condition 	[MAX_CHARBUFFER];
	static int Samples;
    static float UserQoS;

	/* check first if two arguments are passed */
    if (argc != 4)
    {
        strcpy ((*interp).result, "wrong # args" );
        return TCL_ERROR;
    }

    /* 0. Copy argv[1] to MSName                        */
	/* 0. Copy argv[2] to ParticipantIndex				*/
	/* 0. Copy argv[3] to Samples						*/
    /* 1. Retrieve MS parameters from ServiceManager    */
    /* 2. Use QoSMapper to map into User Quality        */
 
    /* 0. */
    strcpy (MSName, argv[1]);
	ParticipantIndex = atoi (argv[2]);
	Samples = atoi ( argv[3]);

	/* generate Condition */
	sprintf ( Condition, "A_Participant = '%s' ", InvitationLetter.Participant2[ParticipantIndex] );

    /* check first MS to redisplay user quality         */
    if ( strcmp ( MSName, VIC ) == 0)
    {
        static VicAppStruct VicApp;
 
        /* 1. */
        GetMSParameterNum ("T_VicInfo", "A_FrameRate", Samples , Condition, &(VicApp.FrameRate));
        VicApp.Resolution = QOS_NO_VALUE;
        GetMSParameterString ("T_VicInfo", "A_Compression", Condition, VicApp.Encoding);
        /*2. */
        VicApp2User ( &VicApp, QOS_MAP_MED, &UserQoS );

    }
 
    if (strcmp (MSName, VAT ) == 0)
    {
        static VatAppStruct VatApp;
 
        /* 1. */
        GetMSParameterNum ("T_VatInfo", "A_Compression", Samples, Condition, &(VatApp.Encoding));
 
        /* 2. */
        VatApp2User ( &VatApp, QOS_MAP_MED, &UserQoS );
 
    }

	sprintf ((*interp).result, "%f", UserQoS );
	return TCL_OK;

}



/****************************************************************************
*
* Procedure:        Tclcmestartup
* Param:            
*                   
* Result:           TCL_OK
*
* Date:             07/19/96
* Last changed:     07/19/96
* Description:      CME startup procedure
*					0) Secure XDisplay --> Tcl-send command 
*					2) Initializes tables ( create and empty )
*					3) Create global database descriptor
*
* Comment:			sock (!global database descriptor )
*
****************************************************************************/

int Tclcmestartup ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{

	/* 0 */
	system ( SECURE_DISPLAY );


	/* 2 */
    build_tables();

	/* 3. */
	/* Connect to CME database */
    sock = DBConnect (DBGetName ());


	return TCL_OK;
}


/****************************************************************************
*
* Procedure:        Tclcmecleanup
* Param:
*
* Result:           TCL_OK
*
* Date:             07/19/96
* Last changed:     07/19/96
* Description:      CME cleanup procedure
*					0) Release global database descriptor
*                   1) Drops CME tables
*                   2) Drops CME database
*
* Comment:			sock (!global database descriptor )
*
****************************************************************************/
 
int Tclcmecleanup ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{

	/* 0. */
    /* Release CME database descriptor */
    DBRelease (sock);

    /* 1. */
	drop_tables();

	/* 2. */
/*
    DBDestroy ();
*/

    return TCL_OK; 
} 




/****************************************************************************
*
* Procedure:        Tclwritewish
* Param:			argv[1]: MSName (e.g. vic )
*					argv[2]: ParticipantIndex: (e.g. 5)
*					argv[3]: QoSValue ( e.g. 3 )
*
* Result:           TCL_OK
* interp->result:	(-1) OK, (0) Error
* Date:             07/20/96
* Last changed:     07/23/96
* Description:      Writes the quality wishes of the receiver 
*					in the senders database after having
*					checked if quality wish has changed
*                   
* Comment:          sock (!global database descriptor )
*					InvitationLetter (global!)
*
****************************************************************************/

int Tclwritewish ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{

	static int ParticipantIndex;
	static char MSName[MAX_CHARBUFFER];	
	static int QoSValue;
	static char TclStatement[TCL_STATEMENT_MAX];
	static int retval;

    /* 0. Copy argv[1] to MSName                        */
    /* 0. Copy argv[2] to ParticipantIndex              */
    /* 0. Copy argv[3] to QoSValue                      */
	/* 1. Check if QoS specification has changed		*/
	/* 2. Write the wish to remote DB if possible		*/

    /* 0. */
    strcpy (MSName, argv[1]);
    ParticipantIndex = atoi (argv[2]);
    QoSValue = atoi ( argv[3]);

	/* Check first if it is my own slider to be changed */
	if (ParticipantIndex == GetMyIndex () )
		return (TCL_OK);

	/* 1.  */
	if (strcmp ( MSName, VIC ) == 0)
	{
		if (InvitationLetter.VicQoS[ParticipantIndex] == QoSValue )
		{
			(*interp).result = 0;		
			return TCL_OK;
		}
		InvitationLetter.VicQoS[ParticipantIndex] = QoSValue;
	}
	if (strcmp ( MSName, VAT ) == 0) 
    {
        if (InvitationLetter.VatQoS[ParticipantIndex] == QoSValue ) 
        {  
            (*interp).result = 0;       
            return TCL_OK; 
        }
		InvitationLetter.VatQoS[ParticipantIndex] = QoSValue;
    }
    if (strcmp ( MSName, WB ) == 0) 
    {
        if (InvitationLetter.WbQoS[ParticipantIndex] == QoSValue ) 
        {  
            (*interp).result = 0;       
            return TCL_OK; 
        }
		InvitationLetter.WbQoS[ParticipantIndex] = QoSValue;
    }


	

	/* 2. */
	/* delete first old wish */
	sprintf (TclStatement, "delete from T_Wish where \
							A_Participant = '%s' AND A_MSName = '%s' ",\
							InvitationLetter.Participant2[GetMyIndex()],\
							MSName );
	retval = msqlQuery ( InvitationLetter.DBsock[ParticipantIndex], TclStatement );
	if ( retval == -1 )
	{
		printf ("Tclwritewish: ERROR msqlQuery %s\n", TclStatement);
		(*interp).result = 0;		
	}
	
	/* add now QoS wish in T_Wish */
	sprintf (TclStatement, "insert into T_Wish values (\
							'%s', '%s', %d )",\
							InvitationLetter.Participant2[GetMyIndex()],\
							MSName,\
							QoSValue );	
	retval = msqlQuery ( InvitationLetter.DBsock[ParticipantIndex], TclStatement ); 
    if ( retval == -1 ) 
    { 
        printf ("Tclwritewish: ERROR msqlQuery %s\n", TclStatement);         
        (*interp).result = 0;        
    }

	return TCL_OK;

}
							





/****************************************************************************
*
* Procedure:        TclbuildDBsockets
* Param:			--
*
* Result:           TCL_OK
*
* Date:             07/22/96
* Last changed:     07/22/96
* Description:      fills the InvitationLetter constructor <DBsock> with
*					the required socket information
*					<socket descriptor> in success
*					<NO_SOCKET> in failure
*
* Comment:          (global structure InvitationLetter is used )
*
****************************************************************************/


int TclbuildDBsockets ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{
    
	char tempbuf [MAX_CHARBUFFER]; 
	char tempbuf2[MAX_CHARBUFFER]; 
    char *token;
    int i, tempsock;
    char PAddress2[MAX_PARTICIPANT_LENGTH]; /* e.g. 128.32.123.12       */
	char Hostname [MAX_CHARBUFFER]; 		/* e.g. pop					*/
	char DBName   [MAX_CHARBUFFER];

    /* build socket connection for all databases of involved participants 	*/
	/* by contacting the corresponding database daemons						*/
    i = 0;
    while ( strcmp (InvitationLetter.Participant2[i], "") != 0 )
    {
        /* Participant format is: <name>@<address> */
        /* Extract <address> from Participant      */
        strcpy (tempbuf, InvitationLetter.Participant2[i]);
        token = strtok ( tempbuf, "@" );
        token = strtok ( NULL, "@");
        strcpy (PAddress2, token);

		/* Extract <hostname> from Participant --> e.g. pop */
		strcpy (tempbuf, InvitationLetter.Participant[i]);
		token = strtok ( tempbuf, "@" );
		token = strtok ( NULL, "@");
		strcpy (tempbuf2, token );
		token = strtok (tempbuf2, ".");
		strcpy (Hostname, tempbuf2);
		sprintf (DBName, "%s%s", CME_PREFIX, Hostname );

		/* Connect now to remote database */
		tempsock =  msqlConnect ( PAddress2); 
		if ( tempsock != -1)
		{
			InvitationLetter.DBsock[i] = tempsock;
			msqlSelectDB ( tempsock , DBName );
		}
		else /* ... error in msqlConnect */
		{
			InvitationLetter.DBsock[i] = NO_SOCKET; 
		}

        i+=1;                                                                     
    }
 
 
    return TCL_OK;

}




/****************************************************************************
*
* Procedure:        TcldestroyDBsockets
* Param:            --
*
* Result:           TCL_OK
*
* Date:             07/22/96
* Last changed:     07/22/96
* Description:      destroys the DBsock's
*                   
*
* Comment:          (global structure InvitationLetter is used )
*
****************************************************************************/

int TcldestroyDBsockets ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{

	int i;

	i = 1;
	while ( strcmp (InvitationLetter.Participant2[i], "") != 0 )
    {
		if ( InvitationLetter.DBsock[i] != NO_SOCKET )
		{
			msqlClose ( InvitationLetter.DBsock[i] );
		}

		i+=1;
	}

	return TCL_OK;

}




/****************************************************************************
*
* Procedure:        Tclqoscontrol
* Param:            --
*
* Result:           TCL_OK
*
* Date:             07/22/96
* Last changed:     07/22/96
* Description:      first invocation of QoS-controling mechanism
*  
* Comment:          (global structure InvitationLetter is used )
*
****************************************************************************/

int Tclqoscontrol ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )	
{

	/* invoke controlling mechanisms for selected ms services */
	/* 1. VIC */
	if ( InvitationLetter.VicPort != 0 )
	{
		ControlMechanism1 ( VIC );

	}

	/* 2. VAT */
	if ( InvitationLetter.VatPort != 0 )
	{
		ControlMechanism1 ( VAT );

	}


	return TCL_OK;	
}




/****************************************************************************
*
* Procedure:        Tclgetqual
* Param:            MSName
*					ParticipantIndex 
*
* Result:           TCL_OK
* (*interp).result  QoSValue of selected service for <participant>
* Date:             07/22/96
* Last changed:     07/22/96
* Description:      get quality parameters for participant with index i
*
* Comment:          (global structure InvitationLetter is used )
*
****************************************************************************/

int Tclgetqual ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{

    static int ParticipantIndex;
    static char MSName[MAX_CHARBUFFER];

    /* 0. Copy argv[1] to MSName                        */
    /* 0. Copy argv[2] to ParticipantIndex              */
    /* 1. get quality parameters for MS 				*/

    /* 0. */
    strcpy (MSName, argv[1]);
    ParticipantIndex = atoi (argv[2]);

	/* 1. */
	if (strcmp ( MSName, VIC ) == 0 )
		sprintf ( (*interp).result, "%d", InvitationLetter.VicQoS[ParticipantIndex] );
	if (strcmp ( MSName, VAT ) == 0 )
		sprintf ( (*interp).result, "%d", InvitationLetter.VatQoS[ParticipantIndex] );
	if (strcmp ( MSName, WB ) == 0 )
		sprintf ( (*interp).result, "%d", InvitationLetter.WbQoS[ParticipantIndex] );

	return TCL_OK;


}


/****************************************************************************
*
* Procedure:        Tclinitwish
* Param:            --
*
* Result:           TCL_OK
* interp->result:   (-1) OK, (0) Error
* Date:             07/23/96
* Last changed:     07/23/96
* Description:      initialize the T_Wish table on the local site
*					with the default values of the InvitationLetter
*
* Comment:          InvitationLetter (global!)
*					sock: local socket connection (global!)
*
****************************************************************************/

int Tclinitwish ( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{

	char TclStatement[TCL_STATEMENT_MAX];
	char tmpStatement[SQL_STATEMENT_MAX];
    int retval;
	int i = 0;

	while ( strcmp (InvitationLetter.Participant2[i], "" ) != 0 )
	{
		/* wishes are only valid for other participants 	*/
		if  (i != GetMyIndex()) 
		{
			/* VIC */
       		/* ... AND if there doesn't exist already a wish    */ 
        	sprintf (tmpStatement, " select * from T_Wish where A_Participant = '%s' \
                                 AND A_MSName = '%s' ",\
                                 InvitationLetter.Participant2[i],\
                                 VIC );
			if (DBRowsExist (sock, tmpStatement) == 0)
			{
				if ( InvitationLetter.VicPort != 0 )
				{
					sprintf (TclStatement, "insert into T_Wish values (\
											'%s', '%s', %d )", \
											InvitationLetter.Participant2[i],\
											VIC,
											InvitationLetter.VicQoS[i] );
					retval = msqlQuery ( sock, TclStatement );
    				if ( retval == -1 )
    				{
        				printf ("Tclinitwish: ERROR msqlQuery %s\n", TclStatement);
        				(*interp).result = 0;
   					}
				}
			}		
								
            /* VAT */ 
           sprintf (tmpStatement, " select * from T_Wish where A_Participant = '%s' \
                                 AND A_MSName = '%s' ",\
                                 InvitationLetter.Participant2[i],\
                                 VAT );  
            if (DBRowsExist (sock, tmpStatement) == 0) 
            { 
            	if ( InvitationLetter.VatPort != 0 )
            	{  
               	 	sprintf (TclStatement, "insert into T_Wish values (\
                   	                     '%s', '%s', %d )", \
                   	                     InvitationLetter.Participant2[i],\
                   	                     VAT,   
                   	                     InvitationLetter.VatQoS[i] ); 
                	retval = msqlQuery ( sock, TclStatement ); 
                	if ( retval == -1 ) 
                	{
                   	 	printf ("Tclinitwish: ERROR msqlQuery %s\n", TclStatement); 
                   	 	(*interp).result = 0; 
                	} 
				}	
			}
		}

		/* next participant */
		i+=1;
	}

    return TCL_OK;

}
