/****************************************************************************
*
* File:             QoSController.c
* Author:           Nikolaos Radouniklis
* Date:             07/22/96
* Last changed:     07/22/96
* Description:      QoSController executes the various QoS controlling mechanisms
*
*
****************************************************************************/

/* system includes */

#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <msql.h>
#include <tcl.h>
#include <tk.h>

/* my includes */
#include "MS_Types.h"
#include "Constants.h"
#include "Database.h"
#include "TclInterp.h"
#include "QoSMapper.h"
#include "ServiceManager.h"
#include "util.h"

/* my defines */


/* type declarations */

/* function declarations */
int VatMapInv ( int CompressionN, char *CompressionS );


/* global variables */

extern int sock;               /* socket descriptor for CME database access */
extern Tcl_Interp *interp;
extern InvitationLetterStruct InvitationLetter;





/****************************************************************************
*
* Function:         ControlMechanism1
* Param:			MSName (e.g. vic)
*					
* 					
* Returns:			-1 (OK) , 0 (Error)
* Date:             07/22/96
* Last changed:     07/22/96
*
* Description:		ControlMechanism1:
*					Compares sending and requested quality
*					--> applies the "democratic" rule
*					
****************************************************************************/

int ControlMechanism1 (char *MSName )
{

	static char Condition[SQL_STATEMENT_MAX];
	static float RequestedQuality;

    static VicAppStruct VicApp;
    static VatAppStruct VatApp;
	static char AppParam[255];
	
	Tcl_Interp  *interpr; /* tcl interpreter for send commands   */



	/* calculate first the requested quality 						*/
	/* apply the "democratic rule: Average of requested quality 	*/

	/* create Condition */	
	sprintf (Condition, " A_MSName = '%s' ", MSName );
	/* create average value */
	DBAvg (sock, "T_Wish", "A_QoS", Condition, &RequestedQuality ); 

	/* HACK!: fill the invitationletter */
	if ( strcmp ( MSName, VIC ) == 0 )
		InvitationLetter.VicQoS[GetMyIndex()] = (int) RequestedQuality;
	if ( strcmp ( MSName, VAT ) == 0 )
		InvitationLetter.VatQoS[GetMyIndex()] = (int) RequestedQuality;


	/* if no requests exist											*/
	if ( RequestedQuality == QOS_NO_VALUE )
		return (0);

	/* Map from User to Application level 							*/
	/* a. VIC */
	if ( strcmp ( MSName, VIC ) == 0 )
	{
		/* Vic Transmit state					*/
		static int IsTransmit = 1;

		/* check if requested quality is 0      */
		if (RequestedQuality == QOS_ZERO )
		{
			if (IsTransmit == 1)
			{
				/* Stop Transmission 				*/
				GetTclInterp ( &interpr );
				Tcl_Eval ( interpr, "Vic.SetRelease");
				
				IsTransmit = 0;
			}
			return (-1);
		}
		
		if ( IsTransmit == 0 )
		{
			/* Retransmit */
			GetTclInterp ( &interpr );
            Tcl_Eval ( interpr, "Vic.SetTransmit");
			IsTransmit = 1;
		}
		/* Retrieve the application parameters 	*/
		VicUser2App ( RequestedQuality, &VicApp ); 
		/* Set application parameters 			*/
    	sprintf ( AppParam, "%f", VicApp.FrameRate );
    	SetMSParameter (VIC , VIC_MAXFPS, AppParam);
    	sprintf ( AppParam, "%f", VicApp.Resolution );
    	SetMSParameter (VIC , VIC_RESOLUTION, AppParam);
	}
	
	/* b. VAT */
	if ( strcmp ( MSName, VAT ) == 0 )
    {
		/* Retrieve the application parameters  */
		VatUser2App ( RequestedQuality, &VatApp);
		/* Set application parameters           */
		VatMapInv (	(int) VatApp.Encoding, AppParam );
		SetMSParameter (VAT, VAT_ENCODING, AppParam );
	}


	return (-1);

}




/****************************************************************************
*
* Function:         VatMapInv
* Param:            [out]  CompressionS (String e.g. GSM, DVI )
*                   [in] CompressionN (Numeric Code of Compression)
* Returns:          (-1) OK, 0 Error
* Date:             07/23/96
* Last changed:     07/23/96
* Description:      Maps Numeric Compression in String Compression
*
*
* Example:          VatMap ( 0 , Char) --> Char: VAT_LPC
*
****************************************************************************/
 
int VatMapInv ( int CompressionN, char *CompressionS )
{
    
	switch ( CompressionN ) 
	{
		case QOS_VAT_PCM:
			strcpy (CompressionS, VAT_PCM);
			break; 
		case QOS_VAT_DVI:
			strcpy (CompressionS, VAT_DVI);
			break; 
		case QOS_VAT_GSM:
			strcpy (CompressionS, VAT_GSM);
			break; 
		case QOS_VAT_LPC:
			strcpy (CompressionS, VAT_LPC);
			break; 
		case QOS_VAT_BAD:
			strcpy (CompressionS, VAT_LPC);
			break; 
		case QOS_VAT_ZERO:
			strcpy (CompressionS, VAT_ZERO);
			break; 
	}

 
    return (-1);
}        
