/****************************************************************************
*
* File:             QoSMapper.h
* Author:           Nikolaos Radouniklis
* Date:             07/01/96
* Last changed:     07/01/96
* Description:      QoS Mapper maps qualities in different levels
*                   1) User level
*                   2) Application level
*                   3) Resource level
*
****************************************************************************/
 
 
#ifndef _qosmapper_h
#define _qosmapper_h


/****************************************************************************
*
* Function:         VicApp2User
* Param:            [in] VicApp (Vic Structure)
*                   -- [in] Encoding (e.g. JPEG)
*                   -- [in] FrameRate  (QOS_NO_VALUE means not taken into consideration)
*                   -- [in] Resolution (QOS_NO_VALUE means not taken into consideration)
*                   [in] MappingStrategy (strategy chosen for mapping)
*                   [out] QoSVic (exact QoS value)
*
* Returns:          Discrete Value between QOS_BAD to QOS_EXCELLENT
* Date:             06/25/96
* Last changed:     07/01/96 (Coefficient Matrix implemented)
* Description:      translates application parameters to user parameters
* Example:          (FrameRate: 15, Resolution: 70) --> Quality: 4, --> QoSValue: 4.45
*
* Comment:          Coefficient matrix for weighing different
*                   application attributes is not implemented yet
*
****************************************************************************/
 
int VicApp2User ( VicAppStruct *VicApp, int MappingStrategy, float* QoSVic);





/****************************************************************************
*
* Function:         VicUser2App
* Param:            [in] QoSLevel (exact QoS value)
*                   [in out] VicApp (structure to be filled with values)
*                   -- [in]  Encoding
*                   -- [out] FrameRate
*                   -- [out] Resolution
*
* Returns:          (-1) OK, (0) Error
* Date:             06/28/96
* Last changed:     07/01/96
* Description:      translates user parameters to application parameters
* Example:          Quality: 4.5 --> (FrameRate: 20, Resolution: 70)
*
*
****************************************************************************/
 
int VicUser2App ( float QoSLevel, VicAppStruct* VicApp );





/****************************************************************************
*
* Function:         VatApp2User
* Param:            [in] VatApp (Vat Structure)
*                   -- [in] Encoding (e.g. DVI, GSM)
*                   [in] MappingStrategy (strategy chosen for mapping)
*                   [out] QoSVat (exact QoS value)
*
* Returns:          Discrete Value between QOS_BAD to QOS_EXCELLENT
* Date:             07/01/96
* Last changed:     07/01/96
* Description:      translates application parameters to user parameters
* Example:          (Encoding Scheme: DVI) --> Quality: 4, --> QoSValue: 4
*
****************************************************************************/
 
int VatApp2User ( VatAppStruct *VatApp, int MappingStrategy, float* QoSVat);






/****************************************************************************
*
* Function:         VatUser2App
* Param:            [in] QoSLevel (exact QoS value)
*                   [out] VatApp (structure to be filled with values)
*                   -- [out]  Encoding
*
* Returns:          (-1) OK, (0) Error
* Date:             07/01/96
* Last changed:     07/01/96
* Description:      translates user parameters to application parameters
* Example:          Quality: 4 --> (Encoding: QOS_VAT_DVI)
*
*
****************************************************************************/
 
int VatUser2App ( float QoSLevel, VatAppStruct* VatApp );










#endif
