/****************************************************************************
*
* File:             ServiceManager.h
* Author:           Nikolaos Radouniklis
* Date:             05/16/96
* Last changed:     07/01/96
* Description:      ServiceManager controls Media Services, Monitors, etc.
*
*
****************************************************************************/

#ifndef _servicemanager_h
#define _servicemanager_h






/****************************************************************************
*
* Function:         StartMS
* Param:            [in] MSName ( Name of the Media Service )
*                   [in] Filename ( Filename of the Media Service )
*                   [in] ArgumentList (e.g. "vic", "crawdad/3456", NULL )
* Returns:          -1 (OK) , 0 (Error)
* Date:             05/24/96
* Last changed:     05/24/96
* Description:      1) Starts the appropriate media service
*                   2) Register media service
*                   3) Starts the corresponding media monitor
*
****************************************************************************/
 
int StartMS ( char *MSName, char *Filename, char *ArgumentList[] );






/****************************************************************************
*
* Function:         StopMS
* Param:            [in] MSName
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/01/96
* Last changed:     06/01/96
* Description:      3) Stops the active media service
*                   2) Unregisters the media service
*                   1) Stops the corresponding media monitor
*
****************************************************************************/
 
                                                                              
int StopMS  (char *MSName);






/****************************************************************************
*
* Function:         GetMSParameterNum
* Param:            [in] TableName
*                   [in] Attr ( Name of attribute)
*                   [in] Samples (number of last samples to be considered)
*                   [in] Condition (e.g. select a certain participant)
*                   [out] Value (value of param)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/06/96
* Last changed:     06/26/96
* Description:      retrieves the parameter value of <Attr>
*                   regarding the last <Samples>
*
****************************************************************************/
 
int GetMSParameterNum (char* Table, char* Attr, int Samples, char* Condition, float *Value);







/****************************************************************************
*
* Function:         GetMSParameterString
* Param:            [in] TableName
*                   [in] Attr ( Name of attribute)
*                   [in] Condition (e.g. select a certain participant)
*                   [out] Value (Value of String)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/26/96
* Last changed:     06/26/96
* Description:      retrieves the parameter string value of <Attr>
*
****************************************************************************/
 
int GetMSParameterString (char* Table, char* Attr, char* Condition, char* Value);







/****************************************************************************
*
* Function:         SetMSParameter
* Param:            [in] MSName (Name of media service)
*                   [in] Attr (Name of attribute)
*                   [in] Value (value of param)
* Returns:          -1 (OK) , 0 (Error)
* Date:             06/24/96
* Last changed:     06/24/96
* Description:      sets the value of the attribute <Attr>
*                   by dispatching to the appropriate set function
*
****************************************************************************/
 
int SetMSParameter (char *MSName, char* Attr, char* Value);






#endif
