/****************************************************************************
*
* File:             TclInterp.c
* Author:           Nikolaos Radouniklis
* Date:             05/31/96
* Last changed:     06/05/96
* Description:      initializes Tcl-interpreter
*
*
****************************************************************************/

/* system includes */

#include <stdio.h>
#include <tcl.h>
#include <tk.h>

/* my includes */
#include "MS_Types.h"
#include "Constants.h"

/* my defines */


/****************************************************************************
*
* Function:         GetTclInterp
* Param:            [out] interpr_adr ( Tcl-interpreter address)
* Returns:          (-1) success, (0) failure
* Date:             06/06/96
* Last changed:     06/06/96
* Description:      create static Tcl-interpreter if not existent
*                   return Tcl-interpreter
*
****************************************************************************/

int GetTclInterp ( Tcl_Interp **interpr_addr )
{

    /* static variable to check existence of Tcl-interpreter */
    static int InterpExists = 0;
    static Tcl_Interp *MyInterpr;
	static Tk_Window tkwin;

    if (InterpExists == 0)
    {

        MyInterpr = Tcl_CreateInterp ();

        /* load procedures located in following files */
        Tcl_EvalFile (MyInterpr, VIC_TCL_INTERFACE);
        Tcl_EvalFile (MyInterpr, VAT_TCL_INTERFACE);

        tkwin = Tk_CreateMainWindow (MyInterpr, NULL , "cme", "Cme");
		/* and hide window since the only purpose is the send command */
		Tcl_Eval ( MyInterpr, "wm withdraw .");

        InterpExists = -1;
    }

    *interpr_addr = MyInterpr;

    return (-1);
}

