#!/usr/local/bin/wish -f
#/****************************************************************************
#*
#* File:             UI.tcl
#* Author:           Nikolaos Radouniklis
#* Date:             07/10/96
#* Last changed:     07/22/96
#* Description:      User Interface for the entire CME session
#*
#* Comment:
#*
#****************************************************************************/

# global variables
# number of display elements
set elem 40.0
# number of quality levels
set qlevels 5.0
# redisplay & adjust interval in ms
set Rinterval 1000
set Ainterval 1000
# number of samples to be considered
set samplerate 1
# default values for vic, vat, wb quality
set defvicqual 3
set defvatqual 5
set defwbqual  1
# wb status
set wbstatus 1



#/****************************************************************************
#*
#* Procedure:        build.cm
#* Param:            
#* Result:           
#*
#* Date:             07/10/96
#* Last changed:     07/10/96
#* Description:      build the cm user interface
#*                   
#****************************************************************************/ 
 
proc build.cm {}  { 

	frame .left 
	frame .right 
	pack .left -side left -padx 4m -pady 4m -anchor nw 
	pack .right -side right -padx 4m -pady 4m -anchor nw
}



#/****************************************************************************
#*
#* Procedure:        build.partlist
#* Param:            [in] w: parent window
#* Result:           
#*
#* Date:             07/10/96
#* Last changed:     07/10/96
#* Description:      build the participant list windows
#*                   
#****************************************************************************/

proc build.partlist w {

	label $w.partlabel  -text "Participant List" -anchor nw
	entry $w.partentry  -width 40 -relief sunken  -justify left -textvariable participant 	
	frame $w.partframe
	frame $w.buttonframe
	pack  $w.partlabel $w.partentry -side top -fill x -anchor nw
	pack  $w.partframe $w.buttonframe -side top -fill x -pady 1m -anchor nw
	
	listbox $w.partframe.participants -width 35 -relief raised \
							-yscrollcommand "$w.partframe.scroll set"
	pack $w.partframe.participants -side left
	scrollbar $w.partframe.scroll -command  "$w.partframe.participants yview"
	pack $w.partframe.scroll -side right -fill y
	
	button $w.buttonframe.add -text Add -command "AddParticipant $w.partframe.participants $w.partentry"
	button $w.buttonframe.delete -text Delete -command "DeleteParticipant $w.partframe.participants"
	button $w.buttonframe.load -text Load -command "LoadDefault $w.partframe.participants" 
	button $w.buttonframe.save -text Save -command "SaveDefault $w.partframe.participants" 
	pack $w.buttonframe.add $w.buttonframe.delete -side left 
	pack $w.buttonframe.save $w.buttonframe.load -side right 

	#create bindings
	bind $w.partentry <Return> " $w.buttonframe.add invoke"

	#initialize partlist
	InitPartList $w.partframe.participants

}
 
#/****************************************************************************
#*
#* Procedure:        build.ms
#* Param:            [in] w: parent window
#* 					 [in] w2: parent window of listbox
#* Result:
#*
#* Date:             07/10/96
#* Last changed:     07/10/96
#* Description:      build the media service windows
#*
#****************************************************************************/

proc build.ms { w w2 } {
	global address
	global ttl

	frame $w.msframe -relief ridge
	frame $w.labelframe 
	label $w.mslabel -text "Media Services" 
	frame $w.vicframe 
	frame $w.vatframe
	frame $w.wbframe
	
	checkbutton $w.vicframe.vicchoice  -text Video -relief flat -variable vic -command "Display vicport vic"
	checkbutton $w.vatframe.vatchoice  -text Audio -relief flat -variable vat -command "Display vatport vat"
	checkbutton $w.wbframe.wbchoice    -text Whiteboard  -relief flat -variable wb -command "Display wbport wb"

	frame $w.addrframe
	label $w.addrframe.addrlabel -text "Address:"
	entry $w.addrframe.address -width 25 -relief sunken -textvariable address 

	frame $w.bottomframe 	
	frame $w.ttlframe -relief sunken 
	frame $w.buttonframe 
	label $w.ttlframe.ttllabel -text "Scope" 
	radiobutton $w.ttlframe.ttlsite -text Site     -variable ttl -value 2   
	radiobutton $w.ttlframe.ttlregion -text Region -variable ttl -value 16 
	radiobutton $w.ttlframe.ttlworld -text World   -variable ttl -value 127  
	entry $w.ttlframe.ttl -width 10 -relief sunken -textvariable ttl 
	button $w.buttonframe.invite -text "Invite" -width 5\
	 	   -command "Invite $w2.partframe.participants" 
	button $w.buttonframe.quit -text "Quit" -width 5 -command {exit.session 0 }
	

	label $w.vicframe.quall -text "QoS:"
	label $w.vatframe.quall -text "QoS:"
	label $w.wbframe.quall -text "QoS:"
	entry $w.vicframe.qualval -width 3 -relief sunken -textvariable vicqual
	entry $w.vatframe.qualval -width 3 -relief sunken -textvariable vatqual
	entry $w.wbframe.qualval -width 3 -relief sunken -textvariable wbqual
	bind $w.vicframe.qualval <FocusOut> { checkqual vic}
	bind $w.vatframe.qualval <FocusOut> { checkqual vat}
	bind $w.wbframe.qualval  <FocusOut> { checkqual wb }

	label $w.vicframe.portl -text "Port:"
	label $w.vatframe.portl -text "Port:"
	label $w.wbframe.portl  -text "Port:"
	entry $w.vicframe.vicport -width 10 -relief sunken -textvariable vicport
	entry $w.vatframe.vatport -width 10 -relief sunken -textvariable vatport
	entry $w.wbframe.wbport -width 10 -relief sunken -textvariable wbport


	pack $w.msframe $w.addrframe $w.bottomframe -side top -fill x -pady 2m
	pack $w.ttlframe $w.buttonframe -in $w.bottomframe -side left -expand 1 -fill both
	pack $w.labelframe $w.vicframe $w.vatframe $w.wbframe  -fill x -side top -in $w.msframe 
	pack $w.mslabel -in $w.labelframe -side left
	
	pack $w.vicframe.vicchoice -side left
	pack $w.vicframe.vicport $w.vicframe.portl $w.vicframe.qualval $w.vicframe.quall -side right
	pack $w.vatframe.vatchoice -side left
	pack $w.vatframe.vatport $w.vatframe.portl $w.vatframe.qualval $w.vatframe.quall -side right
	pack $w.wbframe.wbchoice -side left
	pack $w.wbframe.wbport $w.wbframe.portl $w.wbframe.qualval $w.wbframe.quall -side right


	pack $w.addrframe.addrlabel -side left
	pack $w.addrframe.address -side right
	pack $w.ttlframe.ttllabel $w.ttlframe.ttlsite $w.ttlframe.ttlregion $w.ttlframe.ttlworld\
			$w.ttlframe.ttl -side top -anchor w
	pack $w.buttonframe.quit $w.buttonframe.invite -side bottom -anchor e 


	# display default address
	set address [getdefault address]
	
 
}

#**************************************
#
# Procedure:		checkqual
# Param:			msname (vic,vat,wb)
#
#**************************************

proc checkqual { msname } {
	global vicqual
	global vatqual
	global wbqual
	global defvicqual defvatqual defwbqual

   	#check if quality is in right range
   	if { $msname == "vic" } {
    	if { [regexp {^[0-5]$} $vicqual] == 0 } {
        	open_dialog "Vic Quality value is between: (0 .. 5)"
			set vicqual $defvicqual
        	return
		}
    }
   	if { $msname == "vat" } {
    	if { [regexp {^[0-5]$} $vatqual] == 0 } {
        	open_dialog "Vat tQuality value is between: (0 .. 5)"
			set vatqual $defvatqual
        	return
		}
	}
   	if { $msname == "wb" } {
    	if { [regexp {^[0-1]$} $wbqual] == 0 } {
        	open_dialog "Wb Quality value is between: (0 .. 1)"
			set wbqual $defwbqual
        	return
		}
	}

}





#/****************************************************************************
#*
#* Procedure:        Invite
#* Param:            [in] plist (participant list)
#* Result:
#*
#* Date:             07/11/96
#* Last changed:     07/14/96
#* Description:      Invite people to conference
#*
#****************************************************************************/	

proc Invite { plist } {

	# global variables
	global address
	global vicport vatport wbport vic vat wb
	global ttl port
	global vicqual vatqual wbqual

	# 0. check first if any media are selected
	if { $vic == 0 && $vat == 0 && $wb == 0 } {
		open_dialog "No media services are selected!\n"
		return
	}
	# 0b. check if only one participant is selected
	if { [llength [$plist get 0 end]] <= 1 } {
		open_dialog "No participants are selected!\n"
		return
	}
	# 1. fill the invitation letter
	set failedlist [fillinvitation 	$address $vicport $vatport $wbport \
									$ttl [$plist get 0 end] $port \
									$vicqual $vatqual $wbqual ]

	# 2. check if failedlist is not empty --> prompt failed addresses
	if { [string compare $failedlist ""] != 0 } {
		
		set error_msg "Can't connect to following participants:\n\n"
		set error_msg  $error_msg$failedlist
		set error_rest "\nSpecify participant list again \n"
		set error_msg  $error_msg$error_rest
		open_dialog $error_msg
		return
	}

	# 3. if failedlist is empty ... proceed with session management
	build_session 

}

#/****************************************************************************
#*
#* Procedure:        open_dialog
#* Param:            [in] text (.. to be displayed )
#* Result:
#*
#* Date:             07/15/96
#* Last changed:     07/15/96
#* Description:      display <text> message in window
#*
#****************************************************************************/
proc open_dialog text {

    set w .dialog
    catch "destroy $w"

	toplevel $w
    wm withdraw $w
	wm title $w "CME Message"
 
    message $w.label -text "$text" -borderwidth 2 -relief groove -width 16c
    button $w.button -text OK -command "destroy $w" 
    pack $w.label -expand 1 -fill x -ipadx 4 -ipady 4
    pack $w.button -pady 4
 
    update idletasks
    set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
        - [winfo vrootx [winfo parent $w]]]
    set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
        - [winfo vrooty [winfo parent $w]]]
    wm geom $w +$x+$y
    wm deiconify $w
 
    bind $w <Enter> "focus $w"
}







#/****************************************************************************
#*
#* Procedure:        Display 
#* Param:            [in] portv (textvariable of port)
#*					 [in] button (vic, vat, wb)
#* Result:
#*
#* Date:             07/11/96
#* Last changed:     07/11/96
#* Description:      Display or Hide portnumber
#*
#****************************************************************************/
proc Display { portv button } {
	
	#checkbutton global variables
	global vic
	global vat
	global wb
	#global portnumber ( "vicport", "vatport", "wbport" )
	global $portv 

	set button [set $button]

	if { $button == 1 } {
		set $portv [getdefault $portv]
	}
	if { $button == 0 } {   
        set $portv ""
    }	

}
	




#/****************************************************************************
#*
#* Procedure:        AddParticipant
#* Param:            [in] PList 
#*					 [in] Entry (entry widget)
#* Result:
#*
#* Date:             07/10/96
#* Last changed:     07/10/96
#* Description:		 Add a participant to the list			
#*
#****************************************************************************/

proc AddParticipant { plist entry } {
	global participant

	#check first if particpant address pattern is ok (e.g. NAME@ADDRESS )
	if { [regexp {.+@.+} $participant  ] == 0 } {
		open_dialog "Participant syntax is <name>@<address>\n"
		return
	}
	# check if participant is already in $plist
	if { [lsearch [$plist get 0 end] $participant] != -1} {
		open_dialog "Participant $participant is already in list\n"
		return 
	}
	# check if participant name is empty
	if { $participant == ""} {
		open_dialog "Participant syntax is <name>@<address>\n" 
		return
	}
	
	# insert now participant	
	$plist insert end $participant
	# delete entry widget now
	$entry delete 0 end
	
		
}

#/**************************************************************************** 
#* 
#* Procedure:        DeleteParticipant
#* Param:            [in] PList   
#* Result:
#* 
#* Date:             07/10/96 
#* Last changed:     07/10/96 
#* Description:      Delete a participant from the list          
#* 
#****************************************************************************/	

proc DeleteParticipant { plist } {

	set index [$plist curselection]
	
	#delete only if not 1. Element selected
	if { $index != "" && $index != 0} {
		$plist delete $index
	}

}


#/**************************************************************************** 
#*  
#* Procedure:        LoadDefault
#* Param:            [in] PList    
#* Result: 
#*  
#* Date:             07/10/96  
#* Last changed:     07/10/96  
#* Description:      Load the default list default.lst
#*  
#****************************************************************************/ 

proc LoadDefault { plist } {

	# 1. delete all the participants (except for myself)
	set size [$plist size]
	
	# delete from second element to end
	if { $size > 1 } {	
		$plist delete 1 [expr $size -1]
	}

	# 2. load the profile default.lst
	set f [open ~/.cme.lst]
	while { [gets $f line] >= 0} {
		$plist insert end $line
	}
	close $f
}
	
#/****************************************************************************
#*
#* Procedure:        SaveDefault
#* Param:            [in] PList
#* Result:
#*
#* Date:             07/10/96
#* Last changed:     07/10/96
#* Description:      Save the default list default.lst
#*
#****************************************************************************/

proc SaveDefault { plist } {

	# save the current participant list (except for myself)
	set f [open ~/.cme.lst w]
	set size [$plist size]

	# begin with second element	
	foreach i [$plist get 1 end] {
		puts $f $i
	
	# prompt message
	open_dialog "Participant list is saved \n"
	
	}
	close $f
}


#/****************************************************************************
#*
#* Procedure:        ReadOptions
#* Param:           
#* Result:
#*
#* Date:             07/11/96
#* Last changed:     07/11/96
#* Description:      define the standard options for windows
#*
#****************************************************************************/

proc ReadOptions {} {
	
	#entry options
	option add *Entry.background Gray75
	option add *Entry.foreground Black
	option add *Entry.justify right

	#general options 
	option add *font *-helvetica-bold-r-normal--*-120-*-*-*-*-*-*

}


#/****************************************************************************
#*
#* Procedure:        InitPartList
#* Param:			 [in] PList: Name of the list
#* Result:
#*
#* Date:             07/11/96
#* Last changed:     07/11/96
#* Description:      initialize participant list with your own address
#*
#****************************************************************************/

proc InitPartList {plist} {

	set host [gethostname]
	set user [getusername]

	set participant "$user@$host"
	# insert now participant
    $plist insert end $participant
}



	
#/****************************************************************************
#*
#* Procedure:        InitCM
#* Param:            
#* Result:
#*
#* Date:             07/11/96
#* Last changed:     07/11/96
#* Description:      initialize ConnectionManger
#*
#****************************************************************************/
proc InitCM {} {
	global ttl
	global port
	global vicqual vatqual wbqual
	global defvicqual defvatqual defwbqual

	#1. invoke startup procedure
	cmestartup

	#build the user interface
	ReadOptions
	wm title . "CME Connection Manager"
	build.cm
	build.partlist .left 
	build.ms .right .left

	#set the default time to live
	set ttl 2

	#set vicqual vatqual wbqual
	set vicqual $defvicqual
	set vatqual $defvatqual
	set wbqual $defwbqual

	#set default port number
	set port [getdefault port]
}



#/****************************************************************************
#*
#* Procedure:        build_session
#* Param:			 
#* Result:
#*
#* Date:             07/15/96
#* Last changed:     07/15/96
#* Description:      build the session manager
#*
#****************************************************************************/
proc build_session {}  {

	wm title . "CME Session Manager"
	set wr .right
	set wl .left
	# first destroy the right half of the window
	destroy .right
	destroy .left.buttonframe

	# build now the new user interface 
	# begin with participant list (left side)
	set Plist .left.partframe.participants

	# retrieve the invitation letter
	set Participants [getinvletter plist]

	$Plist delete 0 end
	#build participant list
	foreach i $Participants {
        $Plist insert end $i
	}

	#build new binding for .left.partentry
	bind .left.partentry <Return> { }
	bind $Plist <ButtonRelease-1> { participant_change }

	#build the new right side
	frame $wr 
	pack $wr -side right -padx 4m -pady 4m -anchor nw

	#build scales and QoS displays	
	set vicport [getinvletter vicport]
	set vatport [getinvletter vatport]
	set wbport [getinvletter wbport]

	if { $vicport != 0 } {
		build.service $wr.video Vic 5 
	}
	if { $vatport != 0 } {
		build.service $wr.audio Vat 5 
	}
	if { $wbport != 0 } {
		build.service $wr.wb Wb 1
	}

	#build buttons
	frame $wr.buttonframe -height 100c -relief ridge
	button $wr.buttonframe.exit -text Exit -width 5 -command {exit.session 1}
	button $wr.buttonframe.test -text Start -width 5 -command start.session
	pack $wr.buttonframe -side bottom -anchor e -pady 2m
	pack $wr.buttonframe.exit $wr.buttonframe.test -side bottom -anchor e


	# invoke procedure start.session	
	# start.session
}



#/****************************************************************************
#*
#* Procedure:        build.service 
#* Param:			 [in] w:  window
#*					 [in] s:  service name (Vic, Vat, Wb)
#*					 [in] to: scale to value
#* Result:
#*
#* Date:             07/15/96
#* Last changed:     07/15/96
#* Description:      builds one service (scale&display)
#* 
#***************************************************************************/
proc build.service { w s to } {
	global elem

	frame $w 
	if { $s == "Vic" } {
		scale $w.scale  -label Video  -from 0 -to $to -length 10c -orient horizontal \
    	     -tickinterval 1.0 -sliderlength 6m -showvalue 0 
		bind $w.scale <ButtonRelease> {set.quality vic [.right.video.scale get]}
	} elseif { $s == "Vat" } {
		scale $w.scale  -label Audio  -from 0 -to $to -length 10c -orient horizontal \
    	     -tickinterval 1.0 -sliderlength 6m -showvalue 0 
		bind $w.scale <ButtonRelease> {set.quality vat [.right.audio.scale get]}
	} elseif { $s == "Wb" } {
		        scale $w.scale  -label Whiteboard  -from 0 -to $to -length 10c -orient horizontal \
             -tickinterval 1.0 -sliderlength 6m -showvalue 0 
		bind $w.scale <ButtonRelease> {set.wb [.right.wb.scale get]}
	}

	frame $w.frame -width 10c -height 5m -relief sunken -borderwidth 1
	frame $w.meter -width 5c -height 5m -relief flat -borderwidth 1\
    			   -background blue
	frame $w.compl -width 5c -height 5m  -relief sunken -borderwidth 1
	frame $w.separator -width 10c -height 5m -relief flat
	frame $w.line_separator -width 10c -height 1m -relief sunken -borderwidth 1

	pack $w -side top -fill x -anchor w
	pack $w.scale $w.frame -side top -anchor w -fill x
	# pack $w.meter $w.compl -in $w.frame -side left
	pack $w.separator $w.line_separator -side top -anchor w

	# e.g. build.elem .right.video <framewidth> <no elements>	
	build.elem $w 10 $elem
}

#/****************************************************************************
#*
#* Procedure:        build.elem (elements for scale)
#*
#/****************************************************************************

proc build.elem  { w width noelem } {
	set xpad 0.2
	set elem elem
	set unit c
	set unitm m
    set elem_width [expr (double($width) /$noelem) - (2*$xpad)/10 ]
    set elem_width $elem_width$unit

	set xpad $xpad$unitm
	for { set i 0 } { $i < [expr $noelem - 1]} { incr i} {
		set no $elem$i 
		frame $w.frame.$no 	-width $elem_width -height 5m -relief flat \
							-background grey75 -borderwidth 1
		pack $w.frame.$no -side left -padx $xpad
	}
}

#/**************************************************************************** 
#* 
#* Procedure:        display.elem (elements for scale) 
#* Param:			 [in] $w     (e.g. .right.video.frame
#					 [in] $elem  (e.g. 40 )
#					 [in] $peak  (e.g. 15 --> to be displayed )
#*
#/****************************************************************************
proc display.elem { w elem peak } {

	set text elem
	for { set i 0 } { $i < [expr $elem -1] } {incr i} {
		set no $text$i
		if { $i < [expr $peak - 1]} {
			#display highlight
			$w.$no configure -background tomato4
			# hack
			if { $i == [expr $elem -2]} {
				$w.$no configure -background grey39
			}
		} elseif { $i <  $peak } {
			#display highlight2
			$w.$no configure -background grey39
		} else {
			#display grey
			$w.$no configure -background grey75
		}
	}


}



#/****************************************************************************
#*
#* Procedure:        start.session
#* Param:            
#*  
#* Result:
#*
#* Date:             07/16/96
#* Last changed:     07/16/96
#* Description:      start the cme session (invoked by build_session)
#*
#****************************************************************************/
proc start.session { } {

	global Ainterval

	# build the socket connections for the T_Wish tables
	# --> (socket number is writte in InvitationLetter)
	buildDBsockets

	# initialize wishes (... with default values )
	initwish
	
	# start the media services (started only if in invitationletter)
	startms vic
	startms vat
	startms wb
	
	# display initial values
    set wbport [getinvletter wbport]
    if { $wbport != 0 } {
       .right.wb.scale set 1 
		redisplay_qual wb 5
    }
	
	# recalculate every interval seconds (DISPLAY)
	recalc.session

	# readjust sending parameters every interval seconds (ADJUST)
	adjust.session

}


#/****************************************************************************
#*
#* Procedure:        recalc.session
#* Param:
#*  
#* Result:			
#*
#* Date:             07/16/96
#* Last changed:     07/16/96
#* Description:      recalculate periodically the quality sliders
#*
#****************************************************************************/
proc recalc.session {} {
	global Rinterval
	global samplerate

	
	#redisplay only selectes services (port != 0)
    set vicport [getinvletter vicport]
    set vatport [getinvletter vatport]
    set wbport [getinvletter wbport]	

    set plist .left.partframe.participants
    set index [$plist curselection]
    if { $index != "" } {
		if { $vicport != 0 } {
			update
       		set vicqual [calcqual vic $index $samplerate]
        	redisplay_qual vic $vicqual
		}
		if { $vatport != 0 } {
			update
        	set vatqual [calcqual vat $index $samplerate]
        	redisplay_qual vat $vatqual
		}
    # no participant selected
    } else {
        # empty entry widget and redisplay quality
		if { [.left.partentry get] != "" } {
   			.left.partentry delete 0 end

        	if { $vicport != 0 } {
            	redisplay_qual vic 0
        	}
        	if { $vatport != 0 } {
           	 	redisplay_qual vat 0
        	}
    	}
	}

    after $Rinterval recalc.session
}



#/****************************************************************************
#*
#* Procedure:        adjust.session
#* Param:
#*  
#* Result:          
#*
#* Date:             07/22/96
#* Last changed:     07/22/96
#* Description:      readjust periodically the quality sliders
#*
#****************************************************************************/
proc adjust.session {} {
	global Ainterval


	#call the QoS-Controling mechanism
	update
	# only for vic, vat
	qoscontrol
	update

	after $Ainterval adjust.session
}





#/****************************************************************************
#*
#* Procedure:        set.quality
#* Param:			 msname: (e.g. vic, vat, wb )
#*					 qvalue ( e.g. 0 .. 5)
#*
#* Result:
#*
#* Date:             07/22/96
#* Last changed:     07/22/96
#* Description:      1) set the quality wish in remote database
#*
#****************************************************************************/
proc set.quality { msname qvalue } {
	global participant

	# retrieve the participant index 	
	set plist .left.partframe.participants
    set index [$plist curselection]


	# if participant is selected .. -> proceed
    if { $index != "" } {
		# write my wish in remote database (e.g. writewish <msname> <p> <value>)
		update
		writewish $msname $index $qvalue
		update
	}

}

#/****************************************************************************
#*
#* Procedure:        set.wb
#* Param:            qvalue ( 0 .. 1)  0 = OFF , 1 = ON
#*
#* Result:
#*
#* Date:             07/26/96
#* Last changed:     07/26/96
#* Description:      starts and stops the wb service
#*
#****************************************************************************/
proc set.wb { qvalue } {
	global wbstatus
	set stretch 5
	
	if { ($qvalue == 1) && ($wbstatus == 0) } {
		# start the ms
		startms wb
		redisplay_qual wb [expr $stretch*1]
		set wbstatus 1
	}
	if { ($qvalue == 0) && ($wbstatus == 1) } {
		# stop the ms
		stopms wb
		redisplay_qual wb 0
		set wbstatus 0
	}
}

	


#/****************************************************************************
#*
#* Procedure:        exit.session
#* Param:			 state (0) session was not started, (1) session was started
#*
#* Result:
#*
#* Date:             07/16/96
#* Last changed:     07/16/96
#* Description:      exit the cme session (invoked by Exit button)
#*
#****************************************************************************/
proc exit.session state  {

	# if session was started
	if {$state == 1} {
    	# stop the ms 
		# 1. Stops the Media Services
		# 2. Stops the Media Service Monitors
		# 3. Performs database CME cleanup
    	stopms vic
		stopms vat
		stopms wb

    	# destroy the DBsockets (communication to remote hosts)
    	destroyDBsockets
	}

	#cmecleanup (destroy DB)
	cmecleanup
 
	# exit the program
	exit
}

#/****************************************************************************
#*
#* Procedure:        redisplay_qual
#* Param:			 msname: media service name (e.g. vic, vat, wb)
#*					 value:  quality value to be displayed (e.g. 3.75 )
#* Result:
#*
#* Date:             07/16/96
#* Last changed:     07/18/96
#* Description:      redisplay the quality meters
#*
#****************************************************************************/
proc redisplay_qual { msname value } {

	global elem
	global qlevels
	set peak [expr ($value/$qlevels) * $elem]
	

    if { $msname == "vic" } {
		update
        display.elem .right.video.frame $elem $peak 
		update
    }
    if { $msname == "vat" } {
		update
        display.elem .right.audio.frame $elem $peak 
		update
    }

	if { $msname == "wb" } {
        update
        display.elem .right.wb.frame $elem $peak
        update
    }
}


#/****************************************************************************
#*
#* Procedure:        participant_change
#* Param:            
#*                   
#* Result:
#*
#* Date:             07/22/96
#* Last changed:     07/22/96
#* Description:      what is to do if a new participant is selected?
#*					 0) retrieve My name
#*					 1) synchronize the widget entry
#*					 2) synchronize the slider values (only when its not me)
#* Comment:			 bind procedure to participant listbox
#*
#****************************************************************************/

proc participant_change {} {

	# 0)
    set myhost [gethostname]
    set user   [getusername]
    set ME "$user@$myhost"

	set plist .left.partframe.participants
	set index [$plist curselection]
	set P [$plist get $index $index]
	set IsMe [string match $ME* $P] 

	# 1)
    .left.partentry delete 0 end
    .left.partentry insert 0 [selection get]

	# 2)
    #redisplay only selected services (port != 0)
    set vicport [getinvletter vicport]
    set vatport [getinvletter vatport]
    set wbport [getinvletter wbport]

    if { $index != "" } {
        if { $vicport != 0 } {
			if { $IsMe == 1 } {
				.right.video.scale set [getqual vic $index ]
				.right.video.scale configure -state disabled
			} else {
				.right.video.scale configure -state normal
        		.right.video.scale set [getqual vic $index ]   
			}
        }
        if { $vatport != 0 } {
           if { $IsMe == 1 } { 
                .right.audio.scale set [getqual vat $index ]   
                .right.audio.scale configure -state disabled 
            } else { 
                .right.audio.scale configure -state normal 
                .right.audio.scale set [getqual vat $index ]   
            }
        }
    } 

}


