# *************************************************************************** 
# 
# File:             vic.interface.tcl
# Author:           Nikolaos Radouniklis
# Date:             05/09/96
# Last changed:     06/27/96
# Description:      bunch of Tcl/Tk procedures
#                   to manipulate the vat
#                   audio application v4.0b
# 
# *************************************************************************** 
 




# *************************************************************************** 
#   
# Procedure:        Vat.GetApplicationName
# Date:             05/13/96 
# Param:            - 
# Result:           $ApplicationName
# Description:      Retrieves the Application Name of the currently used vat application 
#                   In version 4.0 the name is "vat"   
# Add Info:         This information is also included in $V(app)
# 
# 
# ***************************************************************************
# 
proc Vat.GetApplicationName { } {
    return vat
}






# ***************************************************************************
#  
# Procedure:        Vat.GetDataNet
# Date:             05/13/96
# Param:            -
# Result:           $DataNet
# Description:      Retrieves the DataNet handle of the currently used vat application 
#                   This information is delivered by $V(data-net)
#                   and can be used to retrieve subsequently address, port and ttl inf. 
#                  	
#  
# ***************************************************************************
#  
proc Vat.GetDataNet { } {

	set App [Vat.GetApplicationName]
	set Cmd "set V(data-net)"
	catch [list send $App $Cmd] DataNet
	return $DataNet

}




# ***************************************************************************
#
# Procedure:        Vat.GetMyAddress
# Date:             05/13/96
# Param:            -
# Result:           $MyAddress
# Description:      Retrieves the internet address of the currently used vat application
#                   This information is delivered by "$V(data-net) addr"
#                   E.g. _o3 addr
#
#
# ***************************************************************************
#
proc Vat.GetMyAddress { } {
 
    set App [Vat.GetApplicationName]
	set DataNet [Vat.GetDataNet]
    set Cmd "$DataNet addr"
    catch [list send $App $Cmd] MyAddress
    return $MyAddress
 
}




# ***************************************************************************
#
# Procedure:        Vat.GetMyPort
# Date:             05/13/96
# Param:            -
# Result:           $MyPort
# Description:      Retrieves the internet port of the currently used vat application
#                   This information is delivered by "$V(data-net) port"
#                   E.g. _o3 port
#
#
# ***************************************************************************
#
proc Vat.GetMyPort { } {
 
    set App [Vat.GetApplicationName]
    set DataNet [Vat.GetDataNet]
    set Cmd "$DataNet port"
    catch [list send $App $Cmd] MyPort  
    return $MyPort
 
}





# ***************************************************************************
#
# Procedure:        Vat.GetMyTTL
# Date:             05/13/96
# Param:            -
# Result:           $MyTTL
# Description:      Retrieves the TTL value of the currently used vat application
#                   This information is delivered by "$V(data-net) ttl"
#                   E.g. _o3 ttl
#
#
# ***************************************************************************
#
proc Vat.GetMyTTL { } {
 
    set App [Vat.GetApplicationName]
    set DataNet [Vat.GetDataNet]
    set Cmd "$DataNet ttl"
    catch [list send $App $Cmd] MyTTL  
    return $MyTTL
 
}








# ***************************************************************************
#
# Procedure:        Vat.SetCtrlMenu
# Date:             05/13/96
# Param:            -
# Result:           -
# Description:      Retrieves the status of the vat control menu (defined/undefined)
#                   If undefined (i.e. window doesn't exist) the procedure
#                   "toggle_window .menu" in "ui-main.tcl" is invoked twice (open/close)
#
#
# ***************************************************************************
#
proc Vat.SetCtrlMenu { } {
 
    set App [Vat.GetApplicationName]
    set Cmd "winfo exists .menu"
    catch [list send $App $Cmd] WindowExists

	if {$WindowExists == 0} {
		set Cmd "toggle_window .menu"
		catch [list send $App $Cmd]
		catch [list send $App $Cmd]
	} 
}







# ***************************************************************************
#
# Procedure:        Vat.GetStatus
# Date:             05/13/96
# Param:            $InfoType: (Listen, Talk) 
# Result:           Status ( 0,1 )
# Description:      Retrieves the listen/talk status of the vat control menu 
#                   
#
#
# ***************************************************************************
#
proc Vat.GetStatus { InfoType } {
 
    set App [Vat.GetApplicationName]
	if { $InfoType == "Listen" } {
    	set Cmd "set unmuted(.m.right.ab.audio.spkr.mute.b)"   
	} elseif { $InfoType == "Talk" } {
		set Cmd "set unmuted(.m.right.ab.audio.mike.mute.b)"
	}

    catch [list send $App $Cmd] Status
 
	return $Status

}
 

 
 
# ***************************************************************************
#
# Procedure:        Vat.SetButton
# Date:             05/13/96
# Param:            $InfoType ( Listen, Talk )
# Result:           -
# Description:      Sets the Listen or Talk button
#                       
#
#
# ***************************************************************************
#
proc Vat.SetButton { InfoType } {
	
	set Button(Listen) spkr
	set Button(Talk)   mike
 
    set App [Vat.GetApplicationName]
	if { [Vat.GetStatus $InfoType ] == 0 } {
		set Cmd ".m.right.ab.audio.$Button($InfoType).mute.b invoke"
    	catch [list send $App $Cmd] 
	}
 
}
 
 



# ***************************************************************************
#
# Procedure:        Vat.ReleaseButton
# Date:             05/13/96 
# Param:            $InfoType ( Listen, Talk )
# Result:           -
# Description:      Releases the Listen or Talk button
#                       
#
#
# ***************************************************************************
#
proc Vat.ReleaseButton { InfoType } {
    
    set Button(Listen) spkr
    set Button(Talk)   mike
 
    set App [Vat.GetApplicationName]
    if { [Vat.GetStatus $InfoType ] == 1 } {
        set Cmd ".m.right.ab.audio.$Button($InfoType).mute.b invoke"
        catch [list send $App $Cmd]
    }

}




# ***************************************************************************
#
# Procedure:        Vat.GetEncoding
# Date:             05/13/96
# Param:            -
# Result:           $EncodingScheme: (PCM, PCM2, PCM4, DVI, DVI2, DVI4, GSM, LPC4)
# Description:      Retrieves the encoding scheme of the active
#                   vat application (Tcl Global Variable: audioFormat)
#
#
# ***************************************************************************
#
proc Vat.GetEncoding { } {

    set App [Vat.GetApplicationName]
    set Cmd "set audioFormat"
    catch [list send $App $Cmd] EncodingScheme
    return $EncodingScheme
}




     

# ***************************************************************************
#
# Procedure:        Vat.SetEncoding
# Date:             05/13/96
# Last Changed:		07/23/96
# Param:            $EncodingScheme (PCM, PCM2, PCM4, DVI, DVI2, DVI4, GSM, LPC4) 
# Result:           
# Description:      Sets the encoding scheme of the active
#                   vat application 
#
#
# ***************************************************************************
#
proc Vat.SetEncoding { EncodingScheme } {

	set Column(pcm)		1
	set Column(pcm2)	1
	set Column(pcm4)	1
	set Column(dvi)		2
	set Column(dvi2)	2
	set Column(dvi4)	2
	set Column(gsm)		3
	set Column(lpc4)	3
    if { $EncodingScheme == "lpc"} {
        set EncodingScheme "lpc4"
    }

    set App [Vat.GetApplicationName]
    set Cmd ".menu.net.frame.me.fmt.p$Column($EncodingScheme).$EncodingScheme invoke"
    catch [list send $App $Cmd] 
}





# *************************************************************************** 
# 
# Procedure:        Vat.GetPriority 
# Date:             05/13/96 
# Param:            
# Result:         	$Priority ( 0.. )
# Description:      Retrieves the priority value of the active
#                   vat application
#
#
# *************************************************************************** 
# 
proc Vat.GetPriority { } {
  
    set App [Vat.GetApplicationName] 
    set Cmd "set V(priority) "
    catch [list send $App $Cmd] Priority

	return $Priority
} 






# ***************************************************************************
#
# Procedure:        Vat.SetPriority
# Date:             05/13/96
# Param:            $Priority: (Value 0.. )
# Result:        
# Description:      Sets the priority value of the active
#                   vat application
#
#
# ***************************************************************************
#
proc Vat.SetPriority { Priority } {
 
    set App [Vat.GetApplicationName]
    set Cmd "set V(priority) $Priority"
    catch [list send $App $Cmd]
}




# ***************************************************************************
#
# Procedure:        Vat.ShowWindow
# Date:             05/13/96
# Param:            $Window : Windowname
# Result:
# Description:      Displays the vat window $Window
#                   E.g. vat top window (.)
#
#
# ***************************************************************************
#
proc Vat.ShowWindow { Window } {
 
    set App [Vat.GetApplicationName]
    set Cmd "wm deiconify $Window"
    catch [list send $App $Cmd]
}




# ***************************************************************************
#
# Procedure:        Vat.HideWindow 
# Date:             05/13/96 
# Param:            $Window : Windowname
# Result: 
# Description:      Hides the vat window $Window
#                   E.g. vat top window (.)
# 
# 
# *************************************************************************** 
# 
proc Vat.HideWindow { Window } {
  
    set App [Vat.GetApplicationName]
    set Cmd "wm withdraw $Window"
    catch [list send $App $Cmd] 
} 








# ***************************************************************************
#
# Procedure:        Vat.FindHandle
# Date:             05/13/96
# Param:            $Participant ( e.g. nikolaus@128.32.201.23)
# Result:			$Handle (e.g. _o6) ; If no Handle available return 0
# Description:      Searches for the corresponding vat handle/object (e.g. _o6)
#                   to a given participant
#
#
# ***************************************************************************
#
proc Vat.FindHandle { Participant } {

	# Initialize handle table #
    set App [Vat.GetApplicationName]
	set Cmd "srctab gen-init"
	catch [list send $App $Cmd] 

	set Handle 0
	# Fetch handles step by step
	set Cmd "srctab gen-next"
	catch [list send $App $Cmd] NextHandle

	while { $NextHandle != "" } {
		set Cmd2 "$NextHandle sdes cname"
		catch [list send $App $Cmd2] NameInternet
		
		# Compare NameInternet with Participant
		if { [string compare $NameInternet $Participant]== 0 } {
			set Handle $NextHandle
		}		
		catch [list send $App $Cmd] NextHandle
	}
	return $Handle
}







# ***************************************************************************
#
# Procedure:        Vat.GetInfo
# Date:             05/13/96
# Last changed:		06/27/96 (added Encoding)
# Param:            $Participant ( e.g. nikolaus@128.32.201.23)
#					$Info: Type of information, e.g. ( Name, Address Email, Encoding)
# Result:           $Value of $Info attribute
# Description:      Retrieves the value of the ordered information type
#                   of a given participant
#
#
# ***************************************************************************
#
proc Vat.GetInfo { Participant Info } {
 
    set App [Vat.GetApplicationName]
	set Handle [Vat.FindHandle $Participant] 
	set Value 0

	if { $Handle == 0 } {
		return ""
	}

	if { $Info == "Name" } {
		set Cmd	"set src_nickname($Handle)"
        catch [list send $App $Cmd] Value
	} elseif { $Info == "Address"} {
		set Cmd "$Handle sdes cname"
        catch [list send $App $Cmd] Value
	} elseif { $Info == "Email"} {
		set Cmd "$Handle sdes email"
        catch [list send $App $Cmd] Value
	} elseif { $Info == "Encoding" } {
		set Cmd "rtp_format $Handle"
		catch [list send $App $Cmd] Value
	}

    return $Value
}






# ***************************************************************************
#
# Procedure:        Vat.GetRTPInfo
# Date:             05/13/96
# Param:            $Participant ( e.g. nikolaus@128.32.201.23)
#                   
# Result:           $RTPInfo (containing Kilobits, Frames, Packets, Loss, ...) 
# Description:      Retrieves a string comprising all the RTP Info at a 
#                   certain time
#
#
# ***************************************************************************
#
proc Vat.GetRTPInfo { Participant } {
 
    set App [Vat.GetApplicationName]    
    set Handle [Vat.FindHandle $Participant]


    set Cmd "$Handle stats"
    catch [list send $App $Cmd] RTPInfo
	
 
    return $RTPInfo
}



# ***************************************************************************
#
# Procedure:        Vat.Quit
# Date:             05/30/96
# Param:            -
# Result:           -
# Description:      Quit Vat Application
#
# ***************************************************************************
#
proc Vat.Quit { } {
 
    set App [Vat.GetApplicationName]
    set Cmd "adios"
    catch [list send $App $Cmd] 
 
}




# ***************************************************************************
#
# Procedure:        Vat.GetParticipantList
# Date:             06/27/96
# Param:            -
# Result:           $ParticipantList (list of active participants)
# Description:      retrieve list of participants
#
# ***************************************************************************
#
proc Vat.GetParticipantList { } {

    set App [Vat.GetApplicationName]
	set ParticipantList ""

    # Initialize handle table #
    set App [Vat.GetApplicationName]
    set Cmd "srctab gen-init"
    catch [list send $App $Cmd]
 
    # Fetch handles step by step
    set Cmd "srctab gen-next"
    catch [list send $App $Cmd] NextHandle
 
    while { $NextHandle != "" } {
        set Cmd2 "$NextHandle sdes cname"
        catch [list send $App $Cmd2] NameInternet

        # append Name to list $ParticipantList, Separator is "|"
        append ParticipantList $NameInternet
        append ParticipantList "|"

     	catch [list send $App $Cmd] NextHandle
     }       

     return $ParticipantList
}


# ***************************************************************************
#
# Procedure:        Vat.IsAlive
# Date:             07/24/96
# Param:            -
# Result:           $Alive (1 = is alive ) ( 0 = is not alive )
# Description:      Check if vat application is alive
#
# ***************************************************************************
#
proc Vat.IsAlive { } {

    set App [Vat.GetApplicationName]
    if { [lsearch [winfo interps] $App] != -1 } {
        # Vat is alive
        set Alive 1
    } else {
        # Vat is dead
        set Alive 0
    }

    return $Alive
}

